//! Ported to Rust from <https://www.shadertoy.com/view/XsXXDn>
//!
//! Original comment:
//! ```glsl
//! // http://www.pouet.net/prod.php?which=57245
//! // If you intend to reuse this shader, please add credits to 'Danilo Guanabara'
//! ```

#![cfg_attr(target_arch = "spirv", no_std)]
#![feature(lang_items)]
#![feature(register_attr)]
#![register_attr(spirv)]

use spirv_std::glam::{vec2, vec4, Vec2, Vec4, Vec4Swizzles};
use spirv_std::storage_class::{Input, Output, UniformConstant};

#[cfg(target_arch = "spirv")]
use spirv_std::num_traits::Float;

pub trait FloatExt {
    fn rem_euclid(self, rhs: Self) -> Self;
}

impl FloatExt for f32 {
    fn rem_euclid(self, rhs: f32) -> f32 {
        let r = self % rhs;
        if r < 0.0 {
            r + rhs.abs()
        } else {
            r
        }
    }
}

pub trait VecExt {
    fn rem_euclid(self, m: f32) -> Self;
}

impl VecExt for Vec2 {
    fn rem_euclid(self, m: f32) -> Vec2 {
        vec2(self.x.rem_euclid(m), self.y.rem_euclid(m))
    }
}

#[allow(unused_attributes)]
#[spirv(fragment)]
pub fn main_fs(
	#[spirv(frag_coord)] frag_coord : Input<Vec4>,
	u_time : UniformConstant<f32>,
	u_resolution : UniformConstant<Vec2>,
	mut output : Output<Vec4>
) {
	let coord = frag_coord.load();
	let resolution = u_resolution.load();
	let time = u_time.load();

    let mut c: [f32; 3] = [0.0, 0.0, 0.0];
    let mut l = 0.0;
    let mut z = time;
    let mut i = 0;
    while i < 3 {
        let mut p = coord.xy() / resolution;
        let mut uv = p;
        p -= Vec2::splat(0.5);
        p.x *= resolution.x / resolution.y;
        z += 0.07;
        l = p.length();
        uv += p / l * (z.sin() + 1.0) * (l * 9.0 - z * 2.0).sin().abs();
        c[i] = 0.01 / (uv.rem_euclid(1.0) - Vec2::splat(0.5)).abs().length();
        i += 1;
    }

    output.store(vec4(c[0] / l, c[1] / l, c[2] / l, time))
}